@@ -1,3 +1,4 @@
+import asyncio
 import time
 import pytz
 import datetime
@@ -5,7 +6,7 @@ import requests
 from libxduauth import EhallSession, SportsSession
 import json
 from datetime import datetime, timedelta
-from typing import List, Dict, Union
+from typing import List, Dict, Union, Tuple
 from pathlib import Path
 import os
 from .data_source import questions_multi, questions_single
@@ -13,6 +14,9 @@ import random
 from nonebot.adapters.onebot.v11 import Message
 from pyDes import des, CBC, PAD_PKCS5
 import binascii
+from requests.cookies import RequestsCookieJar
+import httpx
+from collections import Counter
 
 # 晨午晚检------------------------------------------------------------------------
 
@@ -181,6 +185,7 @@ def get_timetable(ses: EhallSession, username: str,
                         'name': course['name'],
                         'location': course['location'],
                     }
+    # {'2023-01-15':{0:{'name':'', 'location':''}...}}
     with open(os.path.join(basic_path, f"{username}-remake.json"), "w", encoding="utf-8") as f:
 
         f.write(json.dumps(remake, ensure_ascii=False))
@@ -282,7 +287,148 @@ def get_question(mode: int) -> (Message, Message, Message):
         _type = None
     return res, ans, _type
 
-# 加密解密-------------------------------------------------
+
+# 空闲教室查询---------------------------------------------------------------------
+
+def get_teaching_buildings(ses: EhallSession) -> List[Tuple]:
+    datas = ses.post(
+        "https://ehall.xidian.edu.cn/jwapp/sys/kxjas/modules/kxjas/jxlcx.do",
+        data={
+            "*order": "+XXXQDM,+PX,+JXLDM",
+            "querySetting": '[{"name":"XXXQDM","caption":"学校校区","linkOpt":"AND","builderList":"cbl_String","builder":"equal","value":"S","value_display":"南校区"}]',
+            "pageSize": "999",
+        })
+    datas = datas.json()
+    jxl = [(x.get("JXLMC"), x.get("JXLDM"))
+           for x in datas["datas"]["jxlcx"]["rows"]]
+
+    return jxl
+
+
+def get_classroom(ses: EhallSession, build: str) -> List:
+    querySetting = [{"name": "JXLDM", "caption": "教学楼代码",
+                     "builder": "equal", "linkOpt": "AND", "value": build}]
+    rooms = ses.post(
+        "https://ehall.xidian.edu.cn/jwapp/sys/kxjas/modules/kxjas/cxjsqk.do",
+        data={
+            "XNXQDM": "2022-2023-2",
+            "ZC": "1",
+            "XQ": "1",
+            "querySetting": str(querySetting),
+            "*order": "+LC, +JASMC",
+            "pageSize": "999"}).json()
+    return [x["JASMC"] for x in rooms["datas"]
+            ["cxjsqk"]["rows"] if "休息室" not in x["JASMC"]]
+
+async def httpx_client_post(cookies:RequestsCookieJar, url:str, results: Dict[str, List], data:Dict, s:int, e:int, room:str):
+    async with httpx.AsyncClient(cookies=cookies) as client:
+        resp = (await client.post(url, data=data)).json()
+        if not resp["datas"]["xdcxkxjsxq"]["rows"]:
+            results[f"{s}-{e}"].append(room)
+
+async def get_idle_classroom(ses: EhallSession, rooms: List,
+                       time_: str) -> Dict[str, List[str]]:
+    y, m, d = time_.split("-")
+    if 1 <= int(m) <= 7:
+        XN = f'{int(y)-1}-{y}'
+        XQ = '2'
+    else:
+        XN = f'{y}-{int(y)+1}'
+        XQ = '1'
+    response = ses.post(
+        "https://ehall.xidian.edu.cn/jwapp/sys/kxjas/modules/kxjas/rqzhzcjc.do",
+        data={
+            "RQ": time_,
+            "XN": XN,
+            "XQ": XQ,
+        }).json()
+    XQJ = response["datas"]["rqzhzcjc"]["XQJ"]
+    ZC = response["datas"]["rqzhzcjc"]["ZC"]
+    time_sche = [(1, 2), (3, 4), (5, 6), (7, 8), (9, 10)]
+    idle_rooms = {}
+    for s, e in time_sche:
+        idle_rooms[f'{s}-{e}'] = []
+    tasks = []
+    cookies = ses.cookies
+    for room in rooms:
+        for s, e in time_sche:
+            # 同步
+            # response2 = ses.post(
+            #     "https://ehall.xidian.edu.cn/jwapp/sys/kxjas/modules/kxjas/xdcxkxjsxq.do",
+            #     data={
+            #         "JASDM": room,
+            #         "XNXQDM": f"{XN}-{XQ}",
+            #         "KSJC": str(s),
+            #         "JSJC": str(e),
+            #         "ZYLXDM": "01",
+            #         "ZC": str(ZC),
+            #         "XQ": str(XQJ)}).json()
+            # if not response2["datas"]["xdcxkxjsxq"]["rows"]:
+            #     idle_rooms[f"{s}-{e}"].append(room)
+            tasks.append(asyncio.ensure_future(httpx_client_post(cookies=cookies, url="https://ehall.xidian.edu.cn/jwapp/sys/kxjas/modules/kxjas/xdcxkxjsxq.do",results=idle_rooms, data={
+                    "JASDM": room,
+                    "XNXQDM": f"{XN}-{XQ}",
+                    "KSJC": str(s),
+                    "JSJC": str(e),
+                    "ZYLXDM": "01",
+                    "ZC": str(ZC),
+                    "XQ": str(XQJ)},s=s,e=e,room=room)))
+    await asyncio.wait(tasks)
+
+    # 储存方式为{'1-2':['B-102',...]}
+    return idle_rooms
+
+def analyse_best_idle_room(idle_room:Dict[str, List], timetable:Dict[str, Dict[int, Dict[str, str]]], time_:str, building:str)->str:
+    message = ''
+    time_sche = {
+        0:"1-2",
+        1:"3-4",
+        2:"5-6",
+        3:"7-8",
+        4:"9-10",
+        5:"晚自习"
+    }
+    # 如果这一天有课
+    if timetable.get(time_, None):
+        today_course = timetable.get(time_, None)
+        course_locations = [x["location"] for x in today_course.values()]
+        course_buildings = [x.split("-")[0] for x in course_locations]
+        course_rooms = [x.split("-")[1] for x in course_locations]
+        same_building = {}
+        flag = 0
+        for i in range(len(course_buildings)):
+            result = []
+            # 有同一栋楼
+            if course_buildings[i] == building:
+                flag = 1
+                course_time = list(today_course.keys())[i]
+                course_floor = course_rooms[i][0]
+                if course_time != 4:
+                    for room in idle_room[time_sche[course_time+1]]:
+                        if room.split("-")[1][0]  == course_floor:
+                            result.append(room.split("-")[1])
+                for room in idle_room[time_sche[course_time - 1]]:
+                    if room.split("-")[1][0] == course_floor:
+                        result.append(room.split("-")[1])
+                collection_rooms = Counter(result)
+                best_ans = collection_rooms.most_common(1)
+                message += f"结合您{time_}的课表推荐您去{building}的{best_ans[0][0]}教室自习，离您的教室较近且空的时间较多\n"
+    else:
+        message += f"鉴于您{time_}的课表没有课，已为您推荐当天{building}空闲时间最多的教室\n"
+    ans = []
+    for k,v in idle_room.items():
+        ans += v
+    collection_rooms = Counter(ans)
+    best_ans2 = collection_rooms.most_common(5)
+    message += "空闲时间排名前五的教室分别为:\n"
+    for c,t in best_ans2:
+        message += f"{building}的{c}教室，空闲{t}节课\n"
+    message += "祝您学习生活愉快!"
+    return message
+
+
+
+# 加密解密-------------------------------------------------------------------------
 
 
 def des_encrypt(s: str, key: str) -> bytes:
